/**
  ******************************************************************************
  * @file     USART1_CFG.c
  * @author   sinomcu.com-AE
  * @brief    USART1 config
  *    
  *          This file provides functions to usart1 initialization fo RS485 function:
  *              PA2   --> USART1_TX
  *              PA3   <-- USART1_RX
  *          BaudRate:function parameter,8 bit,NONE,STOP 1bit
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
 */

#include "ms32f0xx.h"
#include "user_config.h"
#include "USART1_CFG.h"

#if(defined(EN_USART_DE) && EN_USART_DE)                   /* PA12 RTS for RS485 DE */  
#else                                                      /* GPIO PA12 pull Down ->RS485 IC DE,RX */
/**
  * @brief  set RS485 driver DE pin,output high
  * @param  None
  * @retval None
 */
void gpio_de_recv(void) {
  MS32_GPIO_ResetOutputPin(RS485_DE_Port,RS485_DE_Pin);
}
/**
  * @brief  clear RS485 driver DE pin,output low
  * @param  None
  * @retval None
 */
void gpio_de_send(void) {
  MS32_GPIO_SetOutputPin(RS485_DE_Port,RS485_DE_Pin);
}
#endif

/**
  * @brief  uart IO Initialization
  *    PA2   --> USART1_TX
  *    PA3   <-- USART1_RX
  * @param  None
  * @retval None
 */
void uart_gpio_init(void) {
  MS32_GPIO_InitTypeDef GpioInitStruct = {0};

  /* PA2   --> USART1_TX */
  /* PA3   <-- USART1_RX */
  MS32_GPIO_StructInit(&GpioInitStruct);    
  GpioInitStruct.Pin = MS32_GPIO_PIN_2 |MS32_GPIO_PIN_3;
  GpioInitStruct.Mode = MS32_GPIO_MODE_ALTERNATE;
  GpioInitStruct.Speed = MS32_GPIO_SPEED_FREQ_HIGH;
  GpioInitStruct.OutputType = MS32_GPIO_OUTPUT_PUSHPULL;
  GpioInitStruct.Pull = MS32_GPIO_PULL_UP;
  GpioInitStruct.Alternate = MS32_GPIO_AF_1;
  MS32_GPIO_Init(GPIOA, &GpioInitStruct);                  /* init function had enable clock */

#if(defined(EN_USART_DE) && EN_USART_DE)                   /* PA12 RTS for RS485 DE */  
  GpioInitStruct.Pin = MS32_GPIO_PIN_12;
  GpioInitStruct.Mode = MS32_GPIO_MODE_ALTERNATE;
  GpioInitStruct.Speed = MS32_GPIO_SPEED_FREQ_HIGH;
  GpioInitStruct.OutputType = MS32_GPIO_OUTPUT_PUSHPULL;
  GpioInitStruct.Pull = MS32_GPIO_PULL_UP;
  GpioInitStruct.Alternate = MS32_GPIO_AF_1;
  MS32_GPIO_Init(GPIOA, &GpioInitStruct);
#else                                                      /* GPIO PA12 pull Down ->RS485 IC DE,RX */
  GpioInitStruct.Pin = MS32_GPIO_PIN_12;
  GpioInitStruct.Mode = MS32_GPIO_MODE_OUTPUT;
  GpioInitStruct.OutputType = MS32_GPIO_OUTPUT_PUSHPULL;
  GpioInitStruct.Speed = MS32_GPIO_SPEED_HIGH;
  GpioInitStruct.Pull = MS32_GPIO_PULL_DOWN;
  GpioInitStruct.Alternate = MS32_GPIO_AF_0;
  MS32_GPIO_Init(GPIOA, &GpioInitStruct);
#endif
}

/**
  * @brief  uart1 Initialization Function
  * @param  uint32_t baudrate
  * @retval None
 */
void usart1_rs485_init(uint32_t baudrate) {
  MS32_USART_InitTypeDef UsartInitStruct = {0};

  uart_gpio_init();
  
  /* UART parameter configuration */
  MS32_USART_StructInit(&UsartInitStruct);
  UsartInitStruct.BaudRate = baudrate;
  UsartInitStruct.DataWidth = MS32_USART_DATAWIDTH_8B;
  UsartInitStruct.StopBits = MS32_USART_STOPBITS_1;
  UsartInitStruct.Parity = MS32_USART_PARITY_NONE;
  UsartInitStruct.TransferDirection = MS32_USART_DIRECTION_TX_RX;
  UsartInitStruct.HardwareFlowControl = MS32_USART_HWCONTROL_NONE;
  UsartInitStruct.OverSampling = MS32_USART_OVERSAMPLING_16;
  MS32_USART_Init(USART1, &UsartInitStruct);               /* init function had enable clock */
  
#if(defined(EN_USART_DE) && EN_USART_DE)                   /* Enable DE Mode */  
  MS32_USART_EnableDEMode(USART1);
  MS32_USART_SetDEAssertionTime(USART1,DE_ASSER_TIME);
  MS32_USART_SetDEDeassertionTime(USART1,DE_DASSER_TIME);
#endif
  
  MS32_USART_DisableIT_CTS(USART1);
  MS32_USART_ConfigAsyncMode(USART1);
  MS32_USART_Enable(USART1);

  /*enable USART1 RXNE interrupt*/
  MS32_USART_EnableIT_RXNE(USART1);
  NVIC_EnableIRQ(USART1_IRQn);
  NVIC_SetPriority(USART1_IRQn,0);
}

/******************************** END OF FILE ******************************** */
